@@ -12,7 +12,6 @@
 
 import os
 import time
-import uuid
 
 from cinderclient.v2 import client as cinderclient
 import fixtures
@@ -22,14 +21,16 @@ from keystoneauth1 import identity
 from keystoneauth1 import session as ksession
 from keystoneclient import client as keystoneclient
 from keystoneclient import discover as keystone_discover
+from neutronclient.v2_0 import client as neutronclient
 import os_client_config
-import six
+from oslo_utils import uuidutils
 import tempest.lib.cli.base
 import testtools
 
 import novaclient
 import novaclient.api_versions
 import novaclient.client
+from novaclient.v2 import networks
 import novaclient.v2.shell
 
 BOOT_IS_COMPLETE = ("login as 'cirros' user. default password: "
@@ -80,7 +81,7 @@ def pick_network(networks):
     network_name = os.environ.get('OS_NOVACLIENT_NETWORK')
     if network_name:
         for network in networks:
-            if network.label == network_name:
+            if network.name == network_name:
                 return network
         raise NoNetworkException()
     return networks[0]
@@ -194,7 +195,7 @@ class ClientTestBase(testtools.TestCase):
 
         user = auth_info['username']
         passwd = auth_info['password']
-        tenant = auth_info['project_name']
+        self.project_name = auth_info['project_name']
         auth_url = auth_info['auth_url']
         user_domain_id = auth_info['user_domain_id']
         self.project_domain_id = auth_info['project_domain_id']
@@ -206,7 +207,7 @@ class ClientTestBase(testtools.TestCase):
 
         auth = identity.Password(username=user,
                                  password=passwd,
-                                 project_name=tenant,
+                                 project_name=self.project_name,
                                  auth_url=auth_url,
                                  project_domain_id=self.project_domain_id,
                                  user_domain_id=user_domain_id)
@@ -225,16 +226,22 @@ class ClientTestBase(testtools.TestCase):
         self.image = CACHE["image"]
 
         if "network" not in CACHE:
-            tested_api_version = self.client.api_version
-            proxy_api_version = novaclient.api_versions.APIVersion('2.35')
-            if tested_api_version > proxy_api_version:
-                self.client.api_version = proxy_api_version
-            try:
-                # TODO(mriedem): Get the networks from neutron if using neutron
-                CACHE["network"] = pick_network(self.client.networks.list())
-            finally:
-                self.client.api_version = tested_api_version
+            # Get the networks from neutron.
+            neutron = neutronclient.Client(session=session)
+            neutron_networks = neutron.list_networks()['networks']
+            # Convert the neutron dicts to Network objects.
+            nets = []
+            for network in neutron_networks:
+                nets.append(networks.Network(
+                    networks.NeutronManager, network))
+            # Keep track of whether or not there are multiple networks
+            # available to the given tenant because if so, a specific
+            # network ID has to be passed in on server create requests
+            # otherwise the server POST will fail with a 409.
+            CACHE['multiple_networks'] = len(nets) > 1
+            CACHE["network"] = pick_network(nets)
         self.network = CACHE["network"]
+        self.multiple_networks = CACHE['multiple_networks']
 
         # create a CLI client in case we'd like to do CLI
         # testing. tempest.lib does this really weird thing where it
@@ -248,7 +255,7 @@ class ClientTestBase(testtools.TestCase):
         self.cli_clients = tempest.lib.cli.base.CLIClient(
             username=user,
             password=passwd,
-            tenant_name=tenant,
+            tenant_name=self.project_name,
             uri=auth_url,
             cli_dir=cli_dir,
             insecure=self.insecure)
@@ -258,15 +265,6 @@ class ClientTestBase(testtools.TestCase):
                                               password=passwd)
         self.cinder = cinderclient.Client(auth=auth, session=session)
 
-        if "use_neutron" not in CACHE:
-            # check to see if we're running with neutron or not
-            for service in self.keystone.services.list():
-                if service.type == 'network':
-                    CACHE["use_neutron"] = True
-                    break
-            else:
-                CACHE["use_neutron"] = False
-
     def _get_novaclient(self, session):
         nc = novaclient.client.Client("2", session=session)
 
@@ -375,13 +373,20 @@ class ClientTestBase(testtools.TestCase):
         else:
             self.fail("The resource '%s' still exists." % resource.id)
 
-    def name_generate(self, prefix='Entity'):
-        """Generate randomized name for some entity.
-
-        :param prefix: string prefix
-        """
-        name = "%s-%s" % (prefix, six.text_type(uuid.uuid4()))
-        return name
+    def name_generate(self):
+        """Generate randomized name for some entity."""
+        # NOTE(andreykurilin): name_generator method is used for various
+        #   resources (servers, flavors, volumes, keystone users, etc).
+        #   Since the length of name has limits we cannot use the whole UUID,
+        #   so the first 8 chars is taken from it.
+        #   Based on the fact that the new name includes class and method
+        #   names, 8 chars of uuid should be enough to prevent any conflicts,
+        #   even if the single test will be launched in parallel thousand times
+        return "%(prefix)s-%(test_cls)s-%(test_name)s" % {
+            "prefix": uuidutils.generate_uuid()[:8],
+            "test_cls": self.__class__.__name__,
+            "test_name": self.id().rsplit(".", 1)[-1]
+        }
 
     def _get_value_from_the_table(self, table, key):
         """Parses table to get desired value.
@@ -402,7 +407,8 @@ class ClientTestBase(testtools.TestCase):
                 l_property, l_value = line.split("|")[1:3]
                 if l_property.strip() == key:
                     return l_value.strip()
-        raise ValueError("Property '%s' is missing from the table." % key)
+        raise ValueError("Property '%s' is missing from the table:\n%s" %
+                         (key, table))
 
     def _get_column_value_from_single_row_table(self, table, column):
         """Get the value for the column in the single-row table
@@ -469,14 +475,15 @@ class ClientTestBase(testtools.TestCase):
                     values.append(line.split("|")[1].strip())
         return values
 
-    def _create_server(self, name=None, with_network=True, add_cleanup=True,
-                       **kwargs):
-        name = name or self.name_generate(prefix='server')
+    def _create_server(self, name=None, flavor=None, with_network=True,
+                       add_cleanup=True, **kwargs):
+        name = name or self.name_generate()
         if with_network:
             nics = [{"net-id": self.network.id}]
         else:
             nics = None
-        server = self.client.servers.create(name, self.image, self.flavor,
+        flavor = flavor or self.flavor
+        server = self.client.servers.create(name, self.image, flavor,
                                             nics=nics, **kwargs)
         if add_cleanup:
             self.addCleanup(server.delete)
@@ -485,6 +492,11 @@ class ClientTestBase(testtools.TestCase):
             'building', ['active'])
         return server
 
+    def _wait_for_state_change(self, server_id, status):
+        novaclient.v2.shell._poll_for_status(
+            self.client.servers.get, server_id, None, [status],
+            show_progress=False, poll_period=1, silent=True)
+
     def _get_project_id(self, name):
         """Obtain project id by project name."""
         if self.keystone.version == "v3":
@@ -493,9 +505,20 @@ class ClientTestBase(testtools.TestCase):
             project = self.keystone.tenants.find(name=name)
         return project.id
 
-    def skip_if_neutron(self):
-        if CACHE["use_neutron"]:
-            self.skipTest('nova-network is not available')
+    def _cleanup_server(self, server_id):
+        """Deletes a server and waits for it to be gone."""
+        self.client.servers.delete(server_id)
+        self.wait_for_resource_delete(server_id, self.client.servers)
+
+    def _get_absolute_limits(self):
+        """Returns the absolute limits (quota usage) including reserved quota
+        usage for the given tenant running the test.
+
+        :return: A dict where the key is the limit (or usage) and value.
+        """
+        # The absolute limits are returned in a generator so convert to a dict.
+        return {limit.name: limit.value
+                for limit in self.client.limits.get(reserved=True).absolute}
 
 
 class TenantTestBase(ClientTestBase):
@@ -505,8 +528,8 @@ class TenantTestBase(ClientTestBase):
 
     def setUp(self):
         super(TenantTestBase, self).setUp()
-        user_name = self.name_generate('v' + self.COMPUTE_API_VERSION)
-        project_name = self.name_generate('v' + self.COMPUTE_API_VERSION)
+        user_name = uuidutils.generate_uuid()
+        project_name = uuidutils.generate_uuid()
         password = 'password'
 
         if self.keystone.version == "v3":
