@@ -331,6 +331,14 @@ class FakeSessionClient(base_client.SessionClient):
     #
 
     def get_limits(self, **kw):
+        absolute = {
+            "maxTotalRAMSize": 51200,
+            "maxServerMeta": 5,
+            "maxImageMeta": 5
+        }
+        # 2.57 removes injected_file* entries from the response.
+        if self.api_version < api_versions.APIVersion('2.57'):
+            absolute.update({"maxPersonality": 5, "maxPersonalitySize": 10240})
         return (200, {}, {"limits": {
             "rate": [
                 {
@@ -374,13 +382,7 @@ class FakeSessionClient(base_client.SessionClient):
                     ]
                 }
             ],
-            "absolute": {
-                "maxTotalRAMSize": 51200,
-                "maxServerMeta": 5,
-                "maxImageMeta": 5,
-                "maxPersonality": 5,
-                "maxPersonalitySize": 10240
-            },
+            "absolute": absolute,
         }})
 
     #
@@ -1297,6 +1299,19 @@ class FakeSessionClient(base_client.SessionClient):
     #
 
     def get_os_quota_class_sets_test(self, **kw):
+        # 2.57 removes injected_file* entries from the response.
+        if self.api_version >= api_versions.APIVersion('2.57'):
+            return (200, FAKE_RESPONSE_HEADERS, {
+                'quota_class_set': {
+                    'id': 'test',
+                    'metadata_items': 1,
+                    'ram': 1,
+                    'instances': 1,
+                    'cores': 1,
+                    'key_pairs': 1,
+                    'server_groups': 1,
+                    'server_group_members': 1}})
+
         if self.api_version >= api_versions.APIVersion('2.50'):
             return (200, FAKE_RESPONSE_HEADERS, {
                 'quota_class_set': {
@@ -1329,6 +1344,18 @@ class FakeSessionClient(base_client.SessionClient):
 
     def put_os_quota_class_sets_test(self, body, **kw):
         assert list(body) == ['quota_class_set']
+        # 2.57 removes injected_file* entries from the response.
+        if self.api_version >= api_versions.APIVersion('2.57'):
+            return (200, {}, {
+                'quota_class_set': {
+                    'metadata_items': 1,
+                    'ram': 1,
+                    'instances': 1,
+                    'cores': 1,
+                    'key_pairs': 1,
+                    'server_groups': 1,
+                    'server_group_members': 1}})
+
         if self.api_version >= api_versions.APIVersion('2.50'):
             return (200, {}, {
                 'quota_class_set': {
@@ -1921,26 +1948,31 @@ class FakeSessionClient(base_client.SessionClient):
         return (200, FAKE_RESPONSE_HEADERS, {})
 
     def get_servers_1234_os_instance_actions(self, **kw):
-        return (200, FAKE_RESPONSE_HEADERS, {
-            "instanceActions":
-                [{"instance_uuid": "1234",
+        action = {"instance_uuid": "1234",
                   "user_id": "b968c25e04ab405f9fe4e6ca54cce9a5",
                   "start_time": "2013-03-25T13:45:09.000000",
                   "request_id": "req-abcde12345",
                   "action": "create",
                   "message": None,
-                  "project_id": "04019601fe3648c0abd4f4abfb9e6106"}]})
+                  "project_id": "04019601fe3648c0abd4f4abfb9e6106"}
+        if self.api_version >= api_versions.APIVersion('2.58'):
+            # This is intentionally different from the start_time.
+            action['updated_at'] = '2013-03-25T13:50:09.000000'
+        return (200, FAKE_RESPONSE_HEADERS, {
+            "instanceActions": [action]})
 
     def get_servers_1234_os_instance_actions_req_abcde12345(self, **kw):
+        action = {"instance_uuid": "1234",
+                  "user_id": "b968c25e04ab405f9fe4e6ca54cce9a5",
+                  "start_time": "2013-03-25T13:45:09.000000",
+                  "request_id": "req-abcde12345",
+                  "action": "create",
+                  "message": None,
+                  "project_id": "04019601fe3648c0abd4f4abfb9e6106"}
+        if self.api_version >= api_versions.APIVersion('2.58'):
+            action['updated_at'] = '2013-03-25T13:45:09.000000'
         return (200, FAKE_RESPONSE_HEADERS, {
-            "instanceAction":
-                {"instance_uuid": "1234",
-                 "user_id": "b968c25e04ab405f9fe4e6ca54cce9a5",
-                 "start_time": "2013-03-25T13:45:09.000000",
-                 "request_id": "req-abcde12345",
-                 "action": "create",
-                 "message": None,
-                 "project_id": "04019601fe3648c0abd4f4abfb9e6106"}})
+            "instanceAction": action})
 
     def post_servers_uuid1_action(self, **kw):
         return 202, {}, {}
@@ -2017,6 +2049,10 @@ class FakeSessionClient(base_client.SessionClient):
             migration1.update({"migration_type": "live-migration"})
             migration2.update({"migration_type": "live-migration"})
 
+        if self.api_version >= api_versions.APIVersion("2.59"):
+            migration1.update({"uuid": "11111111-07d5-11e1-90e3-e3dffe0c5983"})
+            migration2.update({"uuid": "22222222-07d5-11e1-90e3-e3dffe0c5983"})
+
         migration_list = []
         instance_uuid = kw.get('instance_uuid', None)
         if instance_uuid == migration1['instance_uuid']:
